﻿<#
.Synopsis
   Runs many components, that check the status of items used by EL. 
.DESCRIPTION
   Invoke-ELHC looks at an Evidence Library 4.0+ installation,
   and determines the relative health via a number of factors,
   including ACL permissions, # of events by category, and health
   of the storage shares used for video retention.
   
.EXAMPLE
  PS c:\> Invoke-ELHC 
  Calculates a composite score and displays the result.

.EXAMPLE
  PS c:\> Invoke-ELHC -Verbose
  This provides verbose output of config values, components, and current status

.NOTES
Author: Dave Wixon
Date: 2/15/18
Version: 1.235

#>
function Invoke-ELHC
{
    #Requires -RunAsAdministrator
    #Requires -Version 4.0
    
    [CmdletBinding(SupportsShouldProcess=$true, 
                  PositionalBinding=$false,
                  ConfirmImpact='Medium')]
    [Alias()]
    [OutputType([PSOBJECT])]

    Param (
        [Parameter(Mandatory=$false, Position=0)]
        [ValidateScript({ Test-Path -Path $_ -PathType File })]
        [string]$ConfigPath = "$PSScriptRoot\ELHC.config"
    )

    $NewVersion = (Find-Module -Repository ELHCRepo -Name ELHealthCheck).version.tostring()
    $CurrentVersion = (Get-InstalledModule -Name ELHealthCheck).Version.ToString()

    #if we are updating, copy over the config and restart ELHC
    if ($NewVersion -gt $CurrentVersion) {
        Find-Module -Repository ELHCRepo -Name ELHealthcheck | Install-Module -Force
		$BasePath = split-path (get-installedmodule ELHealthcheck).InstalledLocation -parent
		Copy-item -path $BasePath\$CurrentVersion\ELHC.config -destination $BasePath\$NewVersion\ELHC.config
        Start-Process powershell.exe -verb runas -ArgumentList '-executionpolicy bypass -command &{invoke-elhc -verbose}'
        exit
    }

    #Load the config and start up ELHC
    $LoadConfig = . {
        Write-Output "`nImporting configuration files...."
        if (Test-Path -Path $ConfigPath) {
            [xml]$ELHCConfig = Get-Content -Path $ConfigPath

        }
        else { 
            Write-Warning -Message "Did not find environmental info.  Get this from a PM"
            Return
        }
    };
    New-ELHC -version $CurrentVersion -timeout 3

    #run sub-functions, storing result hashtables into an array of hashtables
    $RunComponents = . {
        
        #import/dot source the components
        $ImportComponents = . {
            $TotalComponents = 0
            $AvduRows = Get-QueryResult -query "Select * from storage.AvduStorage WHERE SubstationID = '$($ELHCConfig.root.SubstationID)'" -ConnectionString $ELHCConfig.root.ConnectionString 

            Write-Verbose "Importing components...`n"
            $StandaloneComponents = @()
            foreach ($Component in $ELHCConfig.root.components.standalone.component) {
                Write-Verbose "------$($Component.name) : $($Component.enabled)" 
                if ($Component.enabled -eq $true) {
                    $StandaloneComponents += $Component.name
                    $TotalComponents++
                } 
            }
            $AvduComponents = @()
            foreach ($Component in $ELHCConfig.root.components.avdurow.component) {
                Write-Verbose "------$($Component.name) : $($Component.enabled)"
                if ($Component.enabled -eq $true) {
                    $AvduComponents += $Component.name
                    $TotalComponents += $AvduRows.length
                } 
            }
        }
        
        #these components run standalone, not needing any external data to produce output
        $RunStandaloneComponents = . {
            Write-Verbose ""
            Write-Verbose "Running standalone components..."
            
            #somewhere to put the hashtables returned by components
            $ComponentData = @()
            $ComponentCount = 0
            foreach ($Component in $StandaloneComponents) {
                try {
                    #update the host
                    Write-Verbose "------Processing... $($Component)"
                    Write-Progress -Activity "Processing standalone components..." -Status $Component -PercentComplete ($ComponentCount*100/$TotalComponents)
                        
                    $BaseAvduHash = @{
                        'DateGenerated' = get-date -Format 'yyyy-MM-ddTHH:mm:ss.fffzzz'
                        'DepartmentID' = $ELHCConfig.root.DepartmentID
                        'DepartmentName' = $ELHCConfig.root.DepartmentName
                    }
                        
                    #run the component and add its hash output to our boilerplate hash
                    $BaseAvduHash += & $Component -ELHCConfig $ELHCConfig

                    #add the temp hashtable to the main data array
                    $ComponentData += $BaseAvduHash
                    $ComponentCount++
                }
                catch {
                    Write-Warning "An error occurred while processing $($Component.Name): $_"
                }
            }
        }
        
        #these components require a single AvduStorage Row, and produce unique output per row
        $RunAvduComponents = . {
            foreach ($AvduRow in $AvduRows) {
                Write-Verbose ""
                Write-Verbose "Running components on... $($AvduRow.Path)"
                foreach ($Component in $AvduComponents) {
                    try {
                        #update the host
                        Write-Verbose "------Processing... $($Component)"
                        Write-Progress -Activity "Processing share components..." -Status $Component -PercentComplete (($ComponentCount+1)*100/$TotalComponents)
                            
                        #boilerplate info added to each component log output
                        $BaseAvduHash = @{
                            'AvduStorageName' = $AvduRow.Name
                            'AvduStoragePath' = $AvduRow.Path
                            'AvduStorageType' = $AvduRow.Type
                            'AvduStorageQuota' = $AvduRow.QuotaLimit
                            'DateGenerated' = get-date -Format 'yyyy-MM-ddTHH:mm:ss.fffzzz'
                            'DepartmentID' = $ELHCConfig.root.DepartmentID
                            'DepartmentName' = $ELHCConfig.root.DepartmentName
                        }
                        
                        #run the component and add its hash output to our boilerplate hash
                        $BaseAvduHash += & $Component -ELHCConfig $ELHCConfig -AvduRow $AvduRow

                        #add the temp hashtable to the main data array
                        $ComponentData += $BaseAvduHash
                        $ComponentCount++
                        
                    }
                    catch {
                        Write-Warning "An error occurred while processing $($Component): $_"
                    }
                }
            }
        }
    };

    #Iterate through our component output array, posting each to OMS w/unique LogType
    $ConvertAndPOST = . {
        $DateTime = Get-Date -Format 'yyyy-MM-ddThh.mm.ss'
        $LogPath = "$PSScriptRoot\PastOutput\ELHC_$DateTime"
        New-Item -Path "$LogPath.json" -Force
        $c = 0
        foreach ($Log in $ComponentData) {
            Write-Progress -activity "Uploading.... $LogType logs" -PercentComplete (($c*100)/$ComponentData.length)
            $LogType = $Log.LogType
            $Log = ConvertTo-Json $Log -Depth 3
            $Log | Out-File -FilePath "$LogPath.json" -Append
            try {
                $VerbosePreference = 'silentlycontinue'
                $POST = Push-OMSData `
                    -json $Log `
                    -CustomerID $ELHCConfig.root.CustomerID `
                    -SharedKey $ELHCConfig.root.SharedKey `
                    -LogType $LogType `
                    -DateStamp (Get-Date)
                $VerbosePreference = 'continue'
                $c++
            }
            catch {
                Write-Warning "Failed to upload $LogType logs... $_"
                $failure = $true
            }
        }
        if ($failure) { Write-Verbose "`nOne or more logs failed to upload..." }
        else { Write-Verbose "`nCongratulations, your data has been uploaded successfully." }

        $PassedELHC = $true
        foreach ($Log in $ComponentData) {
            if ($Log.AlertBool -eq $true) {
                Write-Warning "$($Log.Logtype) raised an alert"
                Start-Sleep -Milliseconds 500
                $PassedELHC = $false
                $Log | Out-Host
            }
        }
        if ($PassedELHC) {
            Write-Verbose "No alerts were generated."
        }

        Write-Verbose "Component output located at `'$LogPath.json`'"
    }

    $Now = Get-date
    while ((Get-Date) -lt $now.AddMinutes(30)) {
        if ((Read-Host "Exit ELHealthcheck? (y/n)") -eq 'y') {
            exit
        }
        Start-Sleep -Seconds 5
    }

    exit
}

  