﻿<#
    .Synopsis
    Short description
    .DESCRIPTION
    Long description
    .EXAMPLE
    Example of how to use this cmdlet
    .EXAMPLE
    Another example of how to use this cmdlet
    .INPUTS
    Inputs to this cmdlet (if any)
    .OUTPUTS
    Output from this cmdlet (if any)
    .NOTES
    General notes
    .COMPONENT
    The component this cmdlet belongs to
    .ROLE
    The role this cmdlet belongs to
    .FUNCTIONALITY
    The functionality that best describes this cmdlet
#>
function Get-EnvironmentInfo
{
    [CmdletBinding(SupportsShouldProcess=$true, 
                  PositionalBinding=$false,
                  ConfirmImpact='Medium')]
    [Alias()]
    [OutputType([PSOBJECT])]
    
    param (
        [Parameter(Mandatory=$True, ParameterSetName='ELHC')]
        [ValidateNotNullOrEmpty()]
        [PSObject]$ELHCConfig = $false,

        [Parameter(Mandatory=$False, ParameterSetName='External')]
        [ValidateNotNullOrEmpty()]
        [string]$ConnectionString = "data source=(local);initial catalog=WGEvidenceLibrary;integrated security=true;connection lifetime=30;"
    )

    if($ELHCConfig) {
        $ConnectionString = $ELHCConfig.root.ConnectionString
    }

    $InitialELversion = (`
        "SELECT
        top 1 Modified,
        Convert(VARCHAR(255),([Version] / Power(2,24) & 0xff))
        + '.' + Convert(VARCHAR(255),([Version] / Power(2,16) & 0xff))
        + '.' + Convert(VARCHAR(255),([Version] / Power(2,12) & 0xf))
        + '.' +  Convert(VARCHAR(255),([Version] & 0xfff)) as 'Version'
        from [WGEvidenceLibrary].[dbo].[Version]
        order by Modified asc"
    )
    $LatestELversion = (`
        "SELECT
        top 1 Modified,
        Convert(VARCHAR(255),([Version] / Power(2,24) & 0xff))
        + '.' + Convert(VARCHAR(255),([Version] / Power(2,16) & 0xff))
        + '.' + Convert(VARCHAR(255),([Version] / Power(2,12) & 0xf))
        + '.' +  Convert(VARCHAR(255),([Version] & 0xfff)) as 'Version'
        from [WGEvidenceLibrary].[dbo].[Version]
        order by Modified desc"
    )
    
    $OldVersionInfo = (Get-QueryResult -query $InitialELversion -connectionString $ConnectionString )
    $NewVersionInfo = (Get-QueryResult -query $LatestELversion -connectionString $ConnectionString )

    $DBVersionQuery = "select @@version as version"
    $DBVersion = Get-QueryResult -query $DBVersionQuery -connectionString $ConnectionString

    $VehiclesTotalQuery = (
        "select COUNT(*) as vehicles
        from dbo.Vehicles
        where Deleted = 0"
    )
    $VehiclesTotal = Get-QueryResult -query $VehiclesTotalQuery -connectionString $ConnectionString

    $VehiclesRecentQuery = (
        "select COUNT(distinct deviceID) as VehicleCount
        from dbo.RecordingEvent
        join dbo.Vehicles v on v.ID = DeviceID
        where RecordStart > dateadd(day, -30, getutcdate())
        and SubstationID = '$($ELHCConfig.root.SubstationID)'"
    )

    $VehiclesRecent = Get-QueryResult -query $VehiclesRecentQuery -connectionString $ConnectionString

    $VISTAsTotalQuery = (
        "select COUNT(*) as vistas
        from Management.Vista"
    )
    $VISTAsTotal = Get-QueryResult -query $VISTAsTotalQuery -connectionString $ConnectionString

    $VISTAsRecentQuery = (
        "select COUNT(distinct deviceID) as VistaCount
        from dbo.RecordingEvent
        join management.vista v on v.ID = DeviceID
        where RecordStart > dateadd(day, -30, getutcdate())
        and SubstationID = '$($ELHCConfig.root.SubstationID)'"
    )

    $VISTAsRecent = Get-QueryResult -query $VISTAsRecentQuery -connectionString $ConnectionString

    $OfficersTotalQuery = (
        "select COUNT(*) as officers
        from dbo.Officers"
    )
    $OfficersTotal = Get-QueryResult -query $OfficersTotalQuery -connectionString $ConnectionString

    $OfficersRecentQuery = (
        "select COUNT(distinct OfficerID) as OfficerCount
        from dbo.RecordingEvent
        where RecordStart > dateadd(day, -30, getutcdate())
        and SubstationID = '$($ELHCConfig.root.SubstationID)'"
    )

    $OfficersRecent = Get-QueryResult -query $OfficersRecentQuery -connectionString $ConnectionString

    $GetOS = . {
        $VerbosePreference = 'silentlycontinue'
        $OS = (Get-CimInstance win32_operatingsystem).Caption
        $VerbosePreference = 'continue'
    }

    $LastBootup = (gwmi win32_operatingsystem | select LastBootUpTime).LastBootupTime.tostring()
    $LastWinUpdate = (gwmi win32_quickfixengineering |sort installedon -desc)[0].InstalledOn.tostring()
    $SyslogErrors = ((Get-EventLog -LogName System -EntryType Error | select timegenerated | where timegenerated -gt (get-date).adddays(-7)).count)

    $Environment = @{
        'LogType' = 'Environment'
        'PCName' = $env:COMPUTERNAME
        'Username' = $env:USERNAME
        'Domain' = $env:USERDOMAIN
        'PSVersion' = $PSVersionTable.PSVersion.Major
        'SQLVersion' = $DBVersion.version.split('-')[0]
        'VISTAsTotal' = $VISTAsTotal.vistas
        'VISTAsRecent' = $VISTAsRecent.VistaCount
        'VehicleTotal' = $VehiclesTotal.vehicles
        'VehiclesRecent' = $VehiclesRecent.VehicleCount
        'OfficerCount' = $OfficersRecent.OfficerCount
        'OfficerTotal' = $OfficersTotal.officers
        'OS' = $OS
        'InitialInstallDate' = $OldVersionInfo.Modified
        'InitialELVersion' = $OldVersionInfo.Version
        'LatestInstallDate' = $NewVersionInfo.Modified
        'LatestELVersion' = $NewVersionInfo.Version
        'LastBootUpTime' = $LastBootup
        'LastWinUpdate' = $LastWinUpdate
        'SyslogErrorsPastWeek' = $SyslogErrors
    }

    return $Environment
}



