﻿function Get-Performance
{
    [CmdletBinding(SupportsShouldProcess=$true, 
                  PositionalBinding=$false,
                  ConfirmImpact='Medium')]
    [Alias()]
    [OutputType([PSOBJECT])]

    param(
        [Parameter(Mandatory=$True, ParameterSetName='ELHC')]
        [ValidateNotNullOrEmpty()]
        [PSObject]$ELHCConfig = $false,

        [Parameter(Mandatory=$False, ParameterSetName='External')]
        [ValidateNotNullOrEmpty()]
        [int]$Samples = 60,
        
        [Parameter(Mandatory=$False, ParameterSetName='External')]
        [ValidateNotNullOrEmpty()]
        [int]$SampleInterval = 1
    )

    if($ELHCConfig) {
        $Samples = $ELHCConfig.root.components.standalone.component.Performance.Samples
        $SampleInterval = $ELHCConfig.root.components.standalone.component.Performance.SampleInterval
    }
    
    $ctrList = @(
        '\PhysicalDisk(_Total)\Avg. Disk Read Queue Length',
        '\PhysicalDisk(_Total)\Avg. Disk Write Queue Length',
        '\Processor(_Total)\% Processor Time',
        '\memory\% committed bytes in use',
        '\Memory\Pages/sec'
    )
    
    $vars = @(
        $DiskRead,
        $DiskWrite,
        $ProcessorTime,
        $PercentMem,
        $MemPages
    )

    for ($i = 1; $i -le $Samples; $i++) {
        $Data = (Get-Counter -Counter $ctrList -SampleInterval $SampleInterval).CounterSamples.CookedValue
        $vars[0] += $Data[0]
        $vars[1] += $Data[1]
        $vars[2] += $Data[2]
        $vars[3] += $Data[3]
        $vars[4] += $Data[4]
        Write-Progress -Activity "Monitoring performance..." -status "Testing sample $i/$Samples" -PercentComplete ($i*100/$Samples) 
    }

    for ($j = 0; $j -lt $vars.Length; $j++) {
        $vars[$j] = [math]::Round(($vars[$j]/$Samples), 2)
    }

    $Output = @{
        'LogType' = 'Performance'
        'DiskRead' = $vars[0]
        'DiskWrite' = $vars[1]
        'ProcessorTime' = $vars[2]
        'PercentMem' = $vars[3]
        'MemPages' = $vars[4]
        'Samples' = $Samples
        'SampleInterval' = $SampleInterval
    }

    if ($ELHCConfig) {
        $Alerts = $ELHCConfig.root.components.standalone.component.performance.alerts
        $Output += @{
            'AlertPercentMem' = [int]$Alerts.alertpercentmem
            'AlertDiskWrite' = [int]$Alerts.alertdiskwrite
            'AlertDiskRead' = [int]$Alerts.alertdiskread
            'AlertMemPages' = [int]$Alerts.alertmempages
            'AlertProcessor' = [int]$Alerts.alertprocessor
        }
            $AlertBool = $false
        if ($Output.DiskWrite -gt $Output.AlertDiskWrite) {
            $AlertBool = $true
        }
        elseif ($Output.DiskRead -gt $Output.AlertDiskRead) {
            $AlertBool = $true
        }
        elseif ($Output.MemPages -gt $Output.AlertMemPages) {
            $AlertBool = $true
        }
        elseif ($Output.ProcessorTime -gt $Output.AlertProcessor) {
            $AlertBool = $true
        }
        elseif ($Output.PercentMem -gt $Output.AlertPercentMem) {
            $AlertBool = $true
        }

        $Output += @{'AlertBool' = $AlertBool}
    }



    return $Output
}