﻿function Get-PurgeInfo
{
    [CmdletBinding(SupportsShouldProcess=$true, 
                  PositionalBinding=$false,
                  ConfirmImpact='Medium')]
    [Alias()]
    [OutputType([PSOBJECT])]

    param (
        [Parameter(Mandatory=$True, ParameterSetName='ELHC')]
        [ValidateNotNullOrEmpty()]
        [PSObject]$ELHCConfig = $false,

        [Parameter(Mandatory=$False, ParameterSetName='External')]
        [ValidateNotNullOrEmpty()]
        [string]$ConnectionString = "data source=(local);initial catalog=WGEvidenceLibrary;integrated security=true;connection lifetime=30;"
    )

    if($ELHCConfig) {
        $ConnectionString = $ELHCConfig.root.ConnectionString
        $AlertDays = $ELHCConfig.root.components.standalone.component.purgeinfo.alerts.AllowedDays
    }

    $Queries = . {
        $PurgeEligibleQuery = (
        "SELECT re.ID, re.REID, re.EventStatus, re.Created, re.Imported, re.UserOnlineStatus, coalesce(DateDiff(d, re.Imported, getutcdate()), 0) as DaysFromImported, mdeti.Value as EventCategory, mdeta.ID as EventCategoryID, mdrp.OnlineRetentionPolicyID, ocrp.Days as OnlineDays, ors.Name as OnlinePolicy, mdrp.ArchiveRetentionPolicyID, acrp.Days as ArchiveDays, ars.Name as ArchivePolicy, ocrp.Days + acrp.Days as TotalRetention, DateAdd(d, ocrp.Days + acrp.Days, re.Imported) as DeletionDate                                        
        FROM dbo.RecordingEvent re                                        
        JOIN RecordingEvent_XR_MetaDataEventTagItem rexmdeti                                         
        ON rexmdeti.RecordingEventID = re.ID                                        
        JOIN MetaDataEventTagItem mdeti                                        
        ON mdeti.ID = rexmdeti.MetaDataEventTagItemID AND mdeti.MetaDataEventTagTypeID = 21                                        
        JOIN MetaDataEventTagAnswer mdeta                                        
        ON mdeti.Value = mdeta.Caption                                        
        LEFT JOIN MetaDataEventTagAnswerWithRetentionPolicy mdrp                                        
        ON mdrp.MetaDataEventTagAnswerID = mdeta.ID                                        
        LEFT JOIN Storage.CategoryRetentionPolicy ocrp                                        
        on mdrp.OnlineRetentionPolicyID = ocrp.ID                                        
        LEFT JOIN Storage.CategoryRetentionPolicy acrp                                        
        ON mdrp.ArchiveRetentionPolicyID = acrp.ID                                        
        JOIN Storage.RetentionState ors                                        
        ON ors.ID = ocrp.RetentionStateID                                        
        JOIN Storage.RetentionState ars                                        
        ON ars.ID = acrp.RetentionStateID
        LEFT JOIN CaseItemInfo cii
        ON cii.RecordingEventID = re.ID                                      
        WHERE ocrp.RetentionStateID = 1 AND acrp.RetentionStateID IN (1, 2) AND DateAdd(d, ocrp.Days + acrp.Days, re.Imported) < GETUTCDATE()
        AND re.SubstationID = '$($ELHCConfig.root.substationID)'                     
        AND re.UserOnlineStatus = 0 AND cii.RecordingEventID IS NULL
        ORDER BY created DESC, imported DESC"
    )
        $PurgeEligible = (Get-QueryResult -query $PurgeEligibleQuery -connectionString $ConnectionString).ID.count
    
        $LastPurgedQuery = (
        "select top 1 OccurredAt
        from audit.AuditEvent 
        where AuditType = '4'
        group by OccurredAt
        order by max(OccurredAt) desc;"
    )
        $LastPurged = (Get-QueryResult -query $LastPurgedQuery -connectionString $ConnectionString).OccurredAt

        $HoursSincePurgeQuery = (
        "select top 1 DateDiff(HOUR, OccurredAt, GETUTCDATE()) as HoursSincePurge
        from audit.AuditEvent 
        where AuditType = '4'
        group by OccurredAt
        order by max(OccurredAt) desc;"
        )
        $HoursSincePurge = (Get-QueryResult -query $HoursSincePurgeQuery -connectionString $ConnectionString).HoursSincePurge
        $DaysSincePurge = [math]::Round(($HoursSincePurge / 24),2)
    }
    
    $AlertBool = $false
    if ($DaysSincePurge -gt $AlertDays) {
        if ($PurgeEligible -ne 0) {
            $AlertBool = $true
        }
    }

    $Result = @{
        'LogType' = 'PurgeInfo'
        'PurgeEligible' = $PurgeEligible
        'LastPurgeDate' = $LastPurged
        'HoursSincePurge' = $HoursSincePurge
        'DaysSincePurge' = $DaysSincePurge
        'AlertDaysSincePurge' = [int]$AlertDays
        'AlertBool' = $AlertBool

    }

    return $Result
}

